/*
 * TimeTable Viewer
 * Copyright(c)1999 by Hiroaki Sengoku <sengoku@gcd.org>
 * Version 0.1	Jun 19, 1999
 * Version 0.2	Jun 20, 1999
 * Version 0.3	Jun 20, 1999
 * Version 0.4	Jun 21, 1999
 * Version 0.5	Jun 22, 1999	AboutDialog
 * Version 0.6	Jun 25, 1999	SelectTrain, HardButton, PrefsDialog
 * Version 0.7	Jun 26, 1999	scroll bug fix
 * Version 0.8	Jul  3, 1999	Arrive, ShareReg, History, Transfer, StopClock
 * Version 0.9	Jul  8,	1999	multi '>', ContFile, CommentMark, scramble
 * Version 1.0	Jul 31, 1999	split bar, DB, ;;CANCEL, '\' esc, HeadType, LRU
 * Version 1.3  Aug  1, 2001	GPL version
 */
#define DEBUG	/* for debug using HostControl */
#undef DEBUG	/* */
#define PROFILE
#undef PROFILE	/* */

#include <Pilot.h>
#include <stdio.h>
#include <string.h>
#include "callback.h"
#include "Rsc.h"
#if defined(DEBUG) || defined(PROFILE)
#include "HostControl.h"
#ifdef TTV_MAIN
HostFILE *HostFp;
#else
extern HostFILE *HostFp;
#endif
#endif

#define HOUR(h)		((TwelveFlag && (h) > 12) ? ((h)-1) % 12 + 1 : (h))

/* error handling */

typedef enum { privErrNone, privErrNotEnoughSpace, privErrCanNotHappen
	   } PrivErrType;

#ifdef TTV_MAIN
static char *PrivErrString(PrivErrType err);
char *(*PrivErrStringP)(PrivErrType err) = PrivErrString;
static void PrivErrDisplay(char *str, Boolean fatal);
void (*PrivErrDisplayP)(char *str, Boolean fatal) = PrivErrDisplay;
static void PrivErrSetCode(Long err);
void (*PrivErrSetCodeP)(Long err) = PrivErrSetCode;
static void setToday(Boolean manual);
void (*setTodayP)(Boolean manual) = setToday;
static void digit2str(Byte x, char *p, Boolean zero);
void (*digit2strP)(Byte x, char *p, Boolean zero) = digit2str;
static void drawClock(void);
void (*drawClockP)(void) = drawClock;
static VoidHand getAppInfo(DmOpenRef dbP);
VoidHand (*getAppInfoP)(DmOpenRef dbP) = getAppInfo;
#else
extern char *(*PrivErrStringP)(PrivErrType err);
extern void (*PrivErrDisplayP)(char *str, Boolean fatal);
extern void (*PrivErrSetCodeP)(Long err);
#define PrivErrString(err)		(*PrivErrStringP)(err)
#define PrivErrDisplay(str,fatal)	(*PrivErrDisplayP)(str,fatal)
#define PrivErrSetCode(err)		(*PrivErrSetCodeP)(err)
extern char *PrivErrExtraStr;
extern void (*setTodayP)(Boolean manual);
#define setToday(manual)	 	(*setTodayP)(manual)
extern void (*digit2strP)(Byte x, char *p, Boolean zero);
extern void (*drawClockP)(void);
#define digit2str(x,p,zero)		(*digit2strP)(x,p,zero)
#define drawClock();			(*drawClockP)()
extern VoidHand (*getAppInfoP)(DmOpenRef dbP);
#define getAppInfo(dbP)			(*getAppInfoP)(dbP)
#endif

#ifdef TTV_DB
static Boolean loadDB(long recordNum);
Boolean (*loadDBP)(long recordNum) = loadDB;
#else
extern Boolean (*loadDBP)(long recordNum);
#define loadDB(recordNum)		(*loadDBP)(recordNum)
#endif

#ifdef TTV_DIALOG
static void doAboutDialog(void);
void (*doAboutDialogP)(void) = doAboutDialog;
static void doDispPrefsDialog(void);
void (*doDispPrefsDialogP)(void) = doDispPrefsDialog;
static void doListPrefsDialog(void);
void (*doListPrefsDialogP)(void) = doListPrefsDialog;
static void doButtonPrefsDialog(void);
void (*doButtonPrefsDialogP)(void) = doButtonPrefsDialog;
static void setPopup(FormPtr frm, Word id, Word lid, short item);
void (*setPopupP)(FormPtr frm, Word id, Word lid, short item) = setPopup;
static Boolean AlarmDialogHandleEvent(EventPtr e);
Boolean (*AlarmDialogHandleEventP)(EventPtr e) = AlarmDialogHandleEvent;
static short getAlarm(void);
short (*getAlarmP)(void) = getAlarm;
static Boolean setAlarm(short minute);
Boolean (*setAlarmP)(short minute) = setAlarm;
static void setSeparator(Byte *p);
void (*setSeparatorP)(Byte *p) = setSeparator;
#else
extern void (*doAboutDialogP)(void);
extern void (*doDispPrefsDialogP)(void);
extern void (*doListPrefsDialogP)(void);
extern void (*doButtonPrefsDialogP)(void);
extern void (*setPopupP)(FormPtr frm, Word id, Word lid, short item);
extern Boolean (*AlarmDialogHandleEventP)(EventPtr e);
extern short (*getAlarmP)(void);
extern Boolean (*setAlarmP)(short minute);
extern void (*setSeparatorP)(Byte *p);
#define doAboutDialog()			(*doAboutDialogP)()
#define doDispPrefsDialog()		(*doDispPrefsDialogP)()
#define doListPrefsDialog()		(*doListPrefsDialogP)()
#define doButtonPrefsDialog()		(*doButtonPrefsDialogP)()
#define setPopup(frm,id,lid,item)	(*setPopupP)(frm,id,lid,item)
#define getAlarm()			(*getAlarmP)()
#define setAlarm(minute) 		(*setAlarmP)(minute)
#define setSeparator(p)			(*setSeparatorP)(p)
#endif

#define STRLINEMAX	500
#define ExCommentsMax	600
#define AlarmMinuteMax	(19*60)		/* 19 hours */
#define SeparatorMapLen	12

/* preferences */

typedef enum {Never, Rarely, Normal, Mostly, Always} FreqType;

#define MenuItemWidth	120
#define MENU_LEN_MAX	40
#define MENU_NITEMS	8
typedef struct {
    char str[MENU_LEN_MAX];
    long recordNum;
} FileMenuType;

#define HISTORYMAX	20
typedef struct {
    ULong StopClock;
    Boolean ManualSelectFlag;
    short SelItem;
    short SelLine;
} HisAttrType;

#define LRU_MAX		40
typedef struct {
    short next;
    long recordNum;
    short OfsArrive;
    short NarrowMode;
    short NLines;
    Boolean ShowArrivesFlag;
} LRU_Type;

#define MidNight	3	/* 3:00 am */

typedef enum {Errday=-1, Allday, Weekday,
		  Monday, Tuesday, Wednesday, Thursday, Friday,
		  Saturday, Sunday, Holiday,
		  User0, User1, User2, User3, User4,
		  User5, User6, User7, User8, User9,
	  } WeekType;

typedef enum { HeadNone, HeadDepTime, HeadNote, HeadTimeNote } HeadType;

typedef enum { ListOrder_DB_Memo, ListOrder_Memo_DB, ListOrder_Mix,
	       ListOrder_DB_Only, ListOrder_Memo_Only
	   } ListOrderType;
typedef enum { InsertPosAlphabetical, InsertPosTop, InsertPosBot
	   } InsertPosType;

/* form */

typedef struct {
    short itop;
    short ilen;
    char *name;
} StnIndexType;

/* database */

#define TimeTableRecordNumMin	0x10000L

typedef enum {InvertMark, UnMark,
	      SuccMark, PrecMark, MiddleMark, Mark, DotMark, DDotMark,
	      CancelMark,
	      Mark00000, Mark11111=Mark00000+0x1F} CommentMarkType;

#define NoComment	52

typedef struct item {
    Byte minute;	/* 0-59 */
    SByte comment;	/* -1, 0-51, 52 */
    Byte mark;		/* comment mark */
} ItemType;

typedef struct line {	/* NEVER contain Word alignment */
    SByte hour;
    SByte length;	/* at most 60 (< 128) */
    Byte excom_msb;	/* length of extra comments (MSB) */
    Byte excom_lsb;	/* length of extra comments (LSB) */
    struct item items[0];	/* variable length */
} LineType;

typedef struct timetable {
    long recordNum;
    LineType **Lines;
    short LinesLen;
    Byte *LinesBuf;
    char **Comments;
    char *CommentsBuf;
    char **Arrives;
    short ArrivesLen;
    char *ArrivesBuf;
    short NarrowsLen;
    char **Narrows;
} TimeTableType;
#define NarrowsOffset	2	/* all, w/o notes */

typedef struct {	/* ver >= 12 */
    UInt renamedCategories;
    char categoryLabels[dmRecNumCategories][dmCategoryLength];
    Byte categoryUniqIDs[dmRecNumCategories];
    Byte lastUniqID;
    Byte reserved0;
    Word reserved1;
    Word reserved2;
    Byte sortOrder;
    Byte reserved3;
    Word reserved4;
    ULong alarmRecID;
    LocalID alarmDbID;
    UInt alarmCardNo;
} TimeTableAppInfoType;
