#define sndMidiNameLength	32	/* MIDI track name length */

/* Commands for SndPlaySmf */
typedef enum SndSmfCmdEnum {
    sndSmfCmdPlay = 1,		/* play the selection */
    sndSmfCmdDuration		/* get the duration in milliseconds */
} SndSmfCmdEnum;

#define sndSmfPlayAllMilliSec		0xFFFFFFFFUL

typedef struct SndSmfOptionsType {
    /* dwStartMilliSec and dwEndMilliSec are used as inputs to the function
       for sndSmfCmdPlay and as outputs for sndSmfCmdDuration */
    DWord dwStartMilliSec;	/* 0 = start from the beginning */
    DWord dwEndMilliSec;	/* sndSmfPlayAllMilliSec = play entire */
    UInt amplitude;		/* relative volume: 0 - sndMaxAmp */
    Boolean interruptible;	/* if true, sound play will be interrupted */
    DWord reserved;		/* MUST SET TO ZERO BEFORE PASSING */
} SndSmfOptionsType;

typedef struct SndSmfChanRangeType {
    Byte bFirstChan;		/* first MIDI channel (0-15 decimal) */
    Byte bLastChan;		/* last MIDI channel (0-15 decimal) */
} SndSmfChanRangeType;

typedef struct SndCallbackInfoType {
    Ptr funcP;		/* pointer to the callback function or NULL */
    DWord dwUserData;	/* value to be passed in the dwUserData parameter */
} SndCallbackInfoType;

typedef struct SndSmfCallbacksType {
    SndCallbackInfoType completion;	/* completion callback function */
    SndCallbackInfoType	blocking;	/* blocking hook callback function */
    SndCallbackInfoType	reserved;	/* SET ALL FIELDS TO ZERO */
} SndSmfCallbacksType;

/* Midi records found by SndCreateMidiList. */
typedef struct SndMidiListItemType {
    Char name[sndMidiNameLength];	/* including NULL terminator */
    ULong uniqueRecID;
    LocalID dbID;
    UInt cardNo;
} SndMidiListItemType;

typedef struct SndMidiRecHdrType {
    DWord signature;	/* set to sndMidiRecSignature */
    Byte bDataOffset;	/* offset from the beginning of the record */
    Byte reserved;	/* set to zero */
} SndMidiRecHdrType;

#define sysTrapSndPlaySmf		0xA2AD
#define sysTrapSndCreateMidiList	0xA2AE

extern Err SndPlaySmf(void* chanP, SndSmfCmdEnum cmd, BytePtr smfP,
		      SndSmfOptionsType* selP,
		      SndSmfChanRangeType* chanRangeP,
		      SndSmfCallbacksType* callbacksP, Boolean bNoWait)
    SYS_TRAP(sysTrapSndPlaySmf);

extern Boolean SndCreateMidiList(ULong creator, Boolean multipleDBs,
				 WordPtr wCountP, Handle *entHP)
    SYS_TRAP(sysTrapSndCreateMidiList);

#define prefAlarmSoundVolume	(prefAnimationLevel+3)
